"""
Append module search paths for third-party packages to sys.path.

This is stripped down and customized for use in py2app applications
"""

import sys 



_libdir ="/lib/python"+sys .version [:3 ]
_parent ="/".join (__file__ .split ("/")[:-1 ])
if not _parent .endswith (_libdir ):
    _parent +=_libdir 
sys .path .append (_parent +"/site-packages.zip")



sys .path .append (_parent +"/site-packages")

ENABLE_USER_SITE =False 

USER_SITE =None 
USER_BASE =None 


def _import_os ():
    global os 
    import os 


_import_os ()

try :
    basestring 
except NameError :
    basestring =str 


def makepath (*paths ):
    dir =os .path .abspath (os .path .join (*paths ))
    return dir ,os .path .normcase (dir )


for m in sys .modules .values ():
    f =getattr (m ,"__file__",None )
    if isinstance (f ,basestring )and os .path .exists (f ):
        m .__file__ =os .path .abspath (m .__file__ )
del m 



L =[]
_dirs_in_sys_path ={}
dir =dircase =None 
for dir in sys .path :



    dir ,dircase =makepath (dir )
    if dircase not in _dirs_in_sys_path :
        L .append (dir )
        _dirs_in_sys_path [dircase ]=1 

sys .path [:]=L 
del dir ,dircase ,L 
_dirs_in_sys_path =None 


def _init_pathinfo ():
    global _dirs_in_sys_path 
    _dirs_in_sys_path =d ={}
    for dir in sys .path :
        if dir and not os .path .isdir (dir ):
            continue 
        dir ,dircase =makepath (dir )
        d [dircase ]=1 


def addsitedir (sitedir ):
    global _dirs_in_sys_path 
    if _dirs_in_sys_path is None :
        _init_pathinfo ()
        reset =1 
    else :
        reset =0 
    sitedir ,sitedircase =makepath (sitedir )
    if sitedircase not in _dirs_in_sys_path :
        sys .path .append (sitedir )
    try :
        names =os .listdir (sitedir )
    except os .error :
        return 
    names .sort ()
    for name in names :
        if name [-4 :]==os .extsep +"pth":
            addpackage (sitedir ,name )
    if reset :
        _dirs_in_sys_path =None 


def addpackage (sitedir ,name ):
    global _dirs_in_sys_path 
    if _dirs_in_sys_path is None :
        _init_pathinfo ()
        reset =1 
    else :
        reset =0 
    fullname =os .path .join (sitedir ,name )
    try :
        with open (fullname )as f :
            while 1 :
                dir =f .readline ()
                if not dir :
                    break 
                if dir [0 ]=="#":
                    continue 
                if dir .startswith ("import"):
                    exec (dir )
                    continue 
                if dir [-1 ]=="\n":
                    dir =dir [:-1 ]
                dir ,dircase =makepath (sitedir ,dir )
                if dircase not in _dirs_in_sys_path and os .path .exists (dir ):
                    sys .path .append (dir )
                    _dirs_in_sys_path [dircase ]=1 
    except IOError :
        return 
    if reset :
        _dirs_in_sys_path =None 


def _get_path (userbase ):
    version =sys .version_info 

    if sys .platform =="darwin"and getattr (sys ,"_framework",None ):
        return "%s/lib/python/site-packages"%(userbase ,)

    return "%s/lib/python%d.%d/site-packages"%(userbase ,version [0 ],version [1 ])


def _getuserbase ():
    env_base =os .environ .get ("PYTHONUSERBASE",None )
    if env_base :
        return env_base 

    def joinuser (*args ):
        return os .path .expanduser (os .path .join (*args ))

    if getattr (sys ,"_framework",None ):
        return joinuser ("~","Library",sys ._framework ,"%d.%d"%sys .version_info [:2 ])

    return joinuser ("~",".local")


def getuserbase ():
    """Returns the `user base` directory path.

    The `user base` directory can be used to store data. If the global
    variable ``USER_BASE`` is not initialized yet, this function will also set
    it.
    """
    global USER_BASE 
    if USER_BASE is None :
        USER_BASE =_getuserbase ()
    return USER_BASE 


def getusersitepackages ():
    """Returns the user-specific site-packages directory path.

    If the global variable ``USER_SITE`` is not initialized yet, this
    function will also set it.
    """
    global USER_SITE 
    userbase =getuserbase ()

    if USER_SITE is None :
        USER_SITE =_get_path (userbase )

    return USER_SITE 





try :
    import sitecustomize 
except ImportError :
    pass 






if hasattr (sys ,"setdefaultencoding"):
    del sys .setdefaultencoding 

if sys .version_info [0 ]==3 :
    import builtins 

    import _sitebuiltins 

    builtins .help =_sitebuiltins ._Helper ()
    builtins .quit =_sitebuiltins .Quitter ("quit","Ctrl-D (i.e. EOF)")
    builtins .exit =_sitebuiltins .Quitter ("exit","Ctrl-D (i.e. EOF)")



"""
Generic dylib path manipulation
"""

import re 

__all__ =["dylib_info"]

_DYLIB_RE =re .compile (
r"""(?x)
(?P<location>^.*)(?:^|/)
(?P<name>
    (?P<shortname>\w+?)
    (?:\.(?P<version>[^._]+))?
    (?:_(?P<suffix>[^._]+))?
    \.dylib$
)
"""
)


def dylib_info (filename ):
    """
    A dylib name can take one of the following four forms:
        Location/Name.SomeVersion_Suffix.dylib
        Location/Name.SomeVersion.dylib
        Location/Name_Suffix.dylib
        Location/Name.dylib

    returns None if not found or a mapping equivalent to:
        dict(
            location='Location',
            name='Name.SomeVersion_Suffix.dylib',
            shortname='Name',
            version='SomeVersion',
            suffix='Suffix',
        )

    Note that SomeVersion and Suffix are optional and may be None
    if not present.
    """
    is_dylib =_DYLIB_RE .match (filename )
    if not is_dylib :
        return None 
    return is_dylib .groupdict ()



"""
Py2app support for project using sip, which basicly means PyQt and wrappers
for other Qt-based libraries.

This will include all C modules that might be used when you import a package
using sip because we have no way to fine-tune this.

The problem with SIP is that all inter-module depedencies (for example from
PyQt4.Qt to PyQt4.QtCore) are handled in C code and therefore cannot be
detected by the python code in py2app).
"""
from __future__ import absolute_import 

import glob 
import os 
import sys 

import pkg_resources 


class Sip (object ):
    def __init__ (self ):
        self .packages =None 
        self .plugin_dir =None 

    def config (self ):
        if self .packages is not None :
            print ("packages",self .packages )
            return self .packages 

        import os 

        import sipconfig 

        try :
            from PyQt4 import pyqtconfig 

            cfg =pyqtconfig .Configuration ()
            qtdir =cfg .qt_lib_dir 
            sipdir =os .path .dirname (cfg .pyqt_mod_dir )
            self .plugin_dir =os .path .join (cfg .qt_dir ,"plugins")
        except ImportError :
            from PyQt5 .QtCore import QLibraryInfo 

            qtdir =QLibraryInfo .location (QLibraryInfo .LibrariesPath )
            self .plugin_dir =QLibraryInfo .location (QLibraryInfo .PluginsPath )
            sipdir =os .path .dirname (sipconfig .__file__ )

        if not os .path .exists (qtdir ):
            print ("sip: Qtdir %r does not exist"%(qtdir ))

            raise ImportError 



        dyld_library_path =os .environ .get ("DYLD_LIBRARY_PATH","").split (":")

        if qtdir not in dyld_library_path :
            dyld_library_path .insert (0 ,qtdir )
            os .environ ["DYLD_LIBRARY_PATH"]=":".join (dyld_library_path )

        self .packages =set ()

        for fn in os .listdir (sipdir ):
            fullpath =os .path .join (sipdir ,fn )
            if os .path .isdir (fullpath ):
                self .packages .add (fn )
                if fn in ("PyQt4","PyQt5"):


                    for sub in os .listdir (fullpath ):
                        if ".py"not in sub :
                            self .packages .add ("%s.%s"%(fn ,sub .replace (".so","")))

        print ("sip: packages: %s"%(self .packages ,))

        return self .packages 

    def check (self ,cmd ,mf ):
        try :
            packages =self .config ()
        except ImportError :
            return None 

        if "PyQt4.uic"in packages :




            if sys .version_info [0 ]==2 :
                ref ="PyQt4.uic.port_v3"
            else :
                ref ="PyQt4.uic.port_v2"

        if "PyQt5.uic"in packages :

            if sys .version_info [0 ]==2 :
                ref ="PyQt5.uic.port_v3"
            else :
                ref ="PyQt5.uic.port_v2"


            mf .lazynodes [ref ]=None 

        for pkg in packages :
            m =mf .findNode (pkg )
            if m is not None and m .filename is not None :
                break 

        else :
            print ("sip: No sip package used in application")
            return None 

        mf .import_hook ("sip",m )
        m =mf .findNode ("sip")


        for pkg in packages :
            try :
                mf .import_hook (pkg ,m )
            except ImportError as exc :
                print ("WARNING: ImportError in sip recipe ignored: %s"%(exc ,))

        if mf .findNode ("PyQt4")is not None or mf .findNode ("PyQt5")is not None :
            resources =[pkg_resources .resource_filename ("py2app","recipes/qt.conf")]

            for item in cmd .qt_plugins :
                if "/"not in item :
                    item =item +"/*"

                if "*"in item :
                    for path in glob .glob (os .path .join (self .plugin_dir ,item )):
                        rel_path =path [len (self .plugin_dir ):]
                        resources .append (
                        (os .path .dirname ("qt_plugins"+rel_path ),[path ])
                        )
                else :
                    resources .append (
                    (
                    os .path .dirname (os .path .join ("qt_plugins",item )),
                    [os .path .join (self .plugin_dir ,item )],
                    )
                    )

            return {"resources":resources }

        return {}


check =Sip ().check 



"""
dyld emulation
"""

import ctypes 
import os 
import platform 
import sys 
from itertools import chain 

from macholib .dylib import dylib_info 
from macholib .framework import framework_info 

__all__ =["dyld_find","framework_find","framework_info","dylib_info"]

if sys .platform =="darwin"and [
int (x )for x in platform .mac_ver ()[0 ].split (".")[:2 ]
]>=[10 ,16 ]:
    try :
        libc =ctypes .CDLL ("libSystem.dylib")

    except OSError :
        _dyld_shared_cache_contains_path =None 

    else :
        try :
            _dyld_shared_cache_contains_path =libc ._dyld_shared_cache_contains_path 
        except AttributeError :
            _dyld_shared_cache_contains_path =None 

        else :
            _dyld_shared_cache_contains_path .restype =ctypes .c_bool 
            _dyld_shared_cache_contains_path .argtypes =[ctypes .c_char_p ]

            if sys .version_info [0 ]!=2 :
                __dyld_shared_cache_contains_path =_dyld_shared_cache_contains_path 

                def _dyld_shared_cache_contains_path (path ):
                    return __dyld_shared_cache_contains_path (path .encode ())

else :
    _dyld_shared_cache_contains_path =None 



_DEFAULT_FRAMEWORK_FALLBACK =[
os .path .expanduser ("~/Library/Frameworks"),
"/Library/Frameworks",
"/Network/Library/Frameworks",
"/System/Library/Frameworks",
]

_DEFAULT_LIBRARY_FALLBACK =[
os .path .expanduser ("~/lib"),
"/usr/local/lib",
"/lib",
"/usr/lib",
]

if sys .version_info [0 ]==2 :

    def _ensure_utf8 (s ):
        if isinstance (s ,unicode ):
            return s .encode ("utf8")
        return s 

else :

    def _ensure_utf8 (s ):
        if s is not None and not isinstance (s ,str ):
            raise ValueError (s )
        return s 


def _dyld_env (env ,var ):
    if env is None :
        env =os .environ 
    rval =env .get (var )
    if rval is None or rval =="":
        return []
    return rval .split (":")


def dyld_image_suffix (env =None ):
    if env is None :
        env =os .environ 
    return env .get ("DYLD_IMAGE_SUFFIX")


def dyld_framework_path (env =None ):
    return _dyld_env (env ,"DYLD_FRAMEWORK_PATH")


def dyld_library_path (env =None ):
    return _dyld_env (env ,"DYLD_LIBRARY_PATH")


def dyld_fallback_framework_path (env =None ):
    return _dyld_env (env ,"DYLD_FALLBACK_FRAMEWORK_PATH")


def dyld_fallback_library_path (env =None ):
    return _dyld_env (env ,"DYLD_FALLBACK_LIBRARY_PATH")


def dyld_image_suffix_search (iterator ,env =None ):
    """For a potential path iterator, add DYLD_IMAGE_SUFFIX semantics"""
    suffix =dyld_image_suffix (env )
    if suffix is None :
        return iterator 

    def _inject (iterator =iterator ,suffix =suffix ):
        for path in iterator :
            if path .endswith (".dylib"):
                yield path [:-len (".dylib")]+suffix +".dylib"
            else :
                yield path +suffix 
            yield path 

    return _inject ()


def dyld_override_search (name ,env =None ):





    framework =framework_info (name )

    if framework is not None :
        for path in dyld_framework_path (env ):
            yield os .path .join (path ,framework ["name"])



    for path in dyld_library_path (env ):
        yield os .path .join (path ,os .path .basename (name ))


def dyld_executable_path_search (name ,executable_path =None ):



    if name .startswith ("@executable_path/")and executable_path is not None :
        yield os .path .join (
        executable_path ,name [len ("@executable_path/"):]
        )


def dyld_loader_search (name ,loader_path =None ):



    if name .startswith ("@loader_path/")and loader_path is not None :
        yield os .path .join (loader_path ,name [len ("@loader_path/"):])


def dyld_default_search (name ,env =None ):
    yield name 

    framework =framework_info (name )

    if framework is not None :
        fallback_framework_path =dyld_fallback_framework_path (env )

        if fallback_framework_path :
            for path in fallback_framework_path :
                yield os .path .join (path ,framework ["name"])

        else :
            for path in _DEFAULT_FRAMEWORK_FALLBACK :
                yield os .path .join (path ,framework ["name"])

    fallback_library_path =dyld_fallback_library_path (env )
    if fallback_library_path :
        for path in fallback_library_path :
            yield os .path .join (path ,os .path .basename (name ))

    else :
        for path in _DEFAULT_LIBRARY_FALLBACK :
            yield os .path .join (path ,os .path .basename (name ))


def dyld_find (name ,executable_path =None ,env =None ,loader_path =None ):
    """
    Find a library or framework using dyld semantics
    """
    name =_ensure_utf8 (name )
    executable_path =_ensure_utf8 (executable_path )
    for path in dyld_image_suffix_search (
    chain (
    dyld_override_search (name ,env ),
    dyld_executable_path_search (name ,executable_path ),
    dyld_loader_search (name ,loader_path ),
    dyld_default_search (name ,env ),
    ),
    env ,
    ):
        if (
        _dyld_shared_cache_contains_path is not None 
        and _dyld_shared_cache_contains_path (path )
        ):
            return path 
        if os .path .isfile (path ):
            return path 
    raise ValueError ("dylib %s could not be found"%(name ,))


def framework_find (fn ,executable_path =None ,env =None ):
    """
    Find a framework using dyld semantics in a very loose manner.

    Will take input such as:
        Python
        Python.framework
        Python.framework/Versions/Current
    """
    try :
        return dyld_find (fn ,executable_path =executable_path ,env =env )
    except ValueError :
        pass 
    fmwk_index =fn .rfind (".framework")
    if fmwk_index ==-1 :
        fmwk_index =len (fn )
        fn +=".framework"
    fn =os .path .join (fn ,os .path .basename (fn [:fmwk_index ]))
    return dyld_find (fn ,executable_path =executable_path ,env =env )



import os 


def check (cmd ,mf ):
    m =mf .findNode ("zmq")
    if m is None or m .filename is None :
        return None 

    dylibs =os .scandir (os .path .join (m .packagepath [0 ],".dylibs"))
    frameworks =[lib .path for lib in dylibs ]

    return {"frameworks":frameworks }



try :
    from imp import (
    C_BUILTIN ,
    C_EXTENSION ,
    PKG_DIRECTORY ,
    PY_COMPILED ,
    PY_FROZEN ,
    PY_SOURCE ,
    find_module ,
    get_magic ,
    get_suffixes ,
    )

except ImportError :








    import importlib .machinery 
    import importlib .util 
    import os 
    import sys 
    import tokenize 
    from importlib ._bootstrap import _ERR_MSG 

    from _imp import is_builtin ,is_frozen 

    SEARCH_ERROR =0 
    PY_SOURCE =1 
    PY_COMPILED =2 
    C_EXTENSION =3 
    PY_RESOURCE =4 
    PKG_DIRECTORY =5 
    C_BUILTIN =6 
    PY_FROZEN =7 
    PY_CODERESOURCE =8 
    IMP_HOOK =9 

    def get_magic ():
        return importlib .util .MAGIC_NUMBER 

    def get_suffixes ():
        extensions =[
        (s ,"rb",C_EXTENSION )for s in importlib .machinery .EXTENSION_SUFFIXES 
        ]
        source =[(s ,"r",PY_SOURCE )for s in importlib .machinery .SOURCE_SUFFIXES ]
        bytecode =[
        (s ,"rb",PY_COMPILED )for s in importlib .machinery .BYTECODE_SUFFIXES 
        ]

        return extensions +source +bytecode 

    def find_module (name ,path =None ):
        if path is None :
            if is_builtin (name ):
                return (None ,None ,("","",C_BUILTIN ))
            elif is_frozen (name ):
                return (None ,None ,("","",PY_FROZEN ))
            else :
                path =sys .path 

        for entry in path :
            package_directory =os .path .join (entry ,name )
            for suffix in [".py",importlib .machinery .BYTECODE_SUFFIXES [0 ]]:
                package_file_name ="__init__"+suffix 
                file_path =os .path .join (package_directory ,package_file_name )
                if os .path .isfile (file_path ):
                    return None ,package_directory ,("","",PKG_DIRECTORY )

            for suffix ,mode ,type_ in get_suffixes ():
                file_name =name +suffix 
                file_path =os .path .join (entry ,file_name )
                if os .path .isfile (file_path ):
                    break 
            else :
                continue 
            break 
        else :
            raise ImportError (_ERR_MSG .format (name ),name =name )

        encoding =None 
        if "b"not in mode :
            with open (file_path ,"rb")as file :
                encoding =tokenize .detect_encoding (file .readline )[0 ]
        file =open (file_path ,mode ,encoding =encoding )
        return file ,file_path ,(suffix ,mode ,type_ )



"""
Class to read the symbol table from a Mach-O header
"""
from __future__ import with_statement 

import sys 

from macholib .mach_o import (
MH_CIGAM_64 ,
MH_MAGIC_64 ,
dylib_module ,
dylib_reference ,
dylib_table_of_contents ,
nlist ,
nlist_64 ,
relocation_info ,
)

__all__ =["SymbolTable"]

if sys .version_info [0 ]==2 :
    range =xrange 


class SymbolTable (object ):
    def __init__ (self ,macho ,header =None ,openfile =None ):
        if openfile is None :
            openfile =open 
        if header is None :
            header =macho .headers [0 ]
        self .macho_header =header 
        with openfile (macho .filename ,"rb")as fh :
            self .symtab =header .getSymbolTableCommand ()
            self .dysymtab =header .getDynamicSymbolTableCommand ()

            if self .symtab is not None :
                self .nlists =self .readSymbolTable (fh )

            if self .dysymtab is not None :
                self .readDynamicSymbolTable (fh )

    def readSymbolTable (self ,fh ):
        cmd =self .symtab 
        fh .seek (self .macho_header .offset +cmd .stroff )
        strtab =fh .read (cmd .strsize )
        fh .seek (self .macho_header .offset +cmd .symoff )
        nlists =[]

        if self .macho_header .MH_MAGIC in [MH_MAGIC_64 ,MH_CIGAM_64 ]:
            cls =nlist_64 
        else :
            cls =nlist 

        for _i in range (cmd .nsyms ):
            cmd =cls .from_fileobj (fh ,_endian_ =self .macho_header .endian )
            if cmd .n_un ==0 :
                nlists .append ((cmd ,""))
            else :
                nlists .append (
                (
                cmd ,
                strtab [cmd .n_un :strtab .find (b"\x00",cmd .n_un )],
                )
                )
        return nlists 

    def readDynamicSymbolTable (self ,fh ):
        cmd =self .dysymtab 
        nlists =self .nlists 

        self .localsyms =nlists [
        cmd .ilocalsym :cmd .ilocalsym +cmd .nlocalsym 
        ]
        self .extdefsyms =nlists [
        cmd .iextdefsym :cmd .iextdefsym +cmd .nextdefsym 
        ]
        self .undefsyms =nlists [
        cmd .iundefsym :cmd .iundefsym +cmd .nundefsym 
        ]
        if cmd .tocoff ==0 :
            self .toc =None 
        else :
            self .toc =self .readtoc (fh ,cmd .tocoff ,cmd .ntoc )

    def readtoc (self ,fh ,off ,n ):
        fh .seek (self .macho_header .offset +off )
        return [dylib_table_of_contents .from_fileobj (fh )for i in range (n )]

    def readmodtab (self ,fh ,off ,n ):
        fh .seek (self .macho_header .offset +off )
        return [dylib_module .from_fileobj (fh )for i in range (n )]

    def readsym (self ,fh ,off ,n ):
        fh .seek (self .macho_header .offset +off )
        refs =[]
        for _i in range (n ):
            ref =dylib_reference .from_fileobj (fh )
            isym ,flags =divmod (ref .isym_flags ,256 )
            refs .append ((self .nlists [isym ],flags ))
        return refs 

    def readrel (self ,fh ,off ,n ):
        fh .seek (self .macho_header .offset +off )
        return [relocation_info .from_fileobj (fh )for i in range (n )]



import sys 
import os 
from PySide6 .QtWidgets import QApplication ,QWidget ,QLabel ,QPushButton ,QCheckBox ,QComboBox ,QListWidget ,QVBoxLayout ,QHBoxLayout ,QFileDialog ,QMessageBox ,QStyleFactory ,QInputDialog ,QDialog 
from PySide6 .QtCore import Qt 
from PySide6 .QtGui import QPalette ,QColor ,QFont 
import numpy as np 
import h5py 
from neo .rawio import AxonRawIO 
import pyabf 


class SDCombineDatasets (QWidget ):
    def __init__ (self ):
        super ().__init__ ()
        self .setWindowTitle ("SD Combine Datasets/Files")
        self .resize (500 ,700 )


        self .app_name_label =QLabel ("SD Combine Datasets/Files")
        self .app_name_label .setAlignment (Qt .AlignCenter )
        self .app_name_label .setFont (QFont ("Arial",16 ,QFont .Bold ))
        self .email_label =QLabel ("shankar.dutt@anu.edu.au")
        self .email_label .setAlignment (Qt .AlignCenter )
        self .select_folder_button =QPushButton ("Select Folder")
        self .include_subfolders_checkbox =QCheckBox ("Include Subfolders")
        self .extension_dropdown =QComboBox ()
        self .extension_dropdown .addItems ([".dataset.npz",".MLdataset.npz",".abf",".h5",".event_data.npz",".event_fitting.npz"])
        self .file_list =QListWidget ()
        self .file_list .setSelectionMode (QListWidget .SelectionMode .MultiSelection )
        self .select_all_checkbox =QCheckBox ("Select All")
        self .same_duration_checkbox =QCheckBox ("Files have the same duration")
        self .same_duration_checkbox .setChecked (True )
        self .folder_path_label =QLabel ()
        self .folder_path_label .setWordWrap (True )
        self .combine_datasets_button =QPushButton ("Combine Datasets/Files")


        main_layout =QVBoxLayout ()
        main_layout .addWidget (self .app_name_label )
        main_layout .addWidget (self .email_label )
        main_layout .addWidget (self .select_folder_button )
        main_layout .addWidget (self .include_subfolders_checkbox )
        main_layout .addWidget (self .extension_dropdown )
        main_layout .addWidget (self .file_list )
        main_layout .addWidget (self .select_all_checkbox )
        main_layout .addWidget (self .same_duration_checkbox )
        main_layout .addWidget (self .folder_path_label )
        main_layout .addWidget (self .combine_datasets_button )

        self .setLayout (main_layout )


        self .select_folder_button .clicked .connect (self .select_folder )
        self .select_all_checkbox .stateChanged .connect (self .select_all )
        self .combine_datasets_button .clicked .connect (self .combine_datasets )

    def select_folder (self ):
        folder_path =QFileDialog .getExistingDirectory (self ,"Select Folder")
        if folder_path :
            self .file_list .clear ()
            self .folder_path_label .setText (f"Selected Folder: {folder_path }")
            extension =self .extension_dropdown .currentText ()
            if self .include_subfolders_checkbox .isChecked ():
                for root ,dirs ,files in os .walk (folder_path ):
                    for file in files :
                        if file .endswith (extension ):
                            file_path =os .path .relpath (os .path .join (root ,file ),folder_path )
                            self .file_list .addItem (file_path )
            else :
                for file in os .listdir (folder_path ):
                    if file .endswith (extension ):
                        self .file_list .addItem (file )

    def select_all (self ,state ):
        is_checked =self .select_all_checkbox .isChecked ()
        for index in range (self .file_list .count ()):
            item =self .file_list .item (index )
            item .setSelected (is_checked )

    def read_abf_file (self ,file_path ):
        try :

            raw_io =AxonRawIO (filename =file_path )


            raw_io .parse_header ()


            channel_index =0 


            signal_size =raw_io .get_signal_size (block_index =0 ,seg_index =0 )



            data =raw_io .get_analogsignal_chunk (block_index =0 ,seg_index =0 ,i_start =0 ,i_stop =signal_size ,channel_indexes =[channel_index ])


            data =raw_io .rescale_signal_raw_to_float (data ,dtype ='float64',channel_indexes =[channel_index ]).flatten ()


            sampling_rate =raw_io .get_signal_sampling_rate ()
            return data ,sampling_rate 
        except :
            abf =pyabf .ABF (file_path )
            data =abf .sweepY 
            sampling_rate =abf .dataRate 
            return data ,sampling_rate 

    def combine_datasets (self ):
        selected_files =[self .file_list .item (i ).text ()for i in range (self .file_list .count ())if self .file_list .item (i ).isSelected ()]
        if selected_files :
            if self .select_all_checkbox .isChecked ():
                selected_files .sort ()
            else :
                selected_indexes =self .file_list .selectedIndexes ()
                selected_files =[self .file_list .item (index .row ()).text ()for index in selected_indexes ]

            extension =self .extension_dropdown .currentText ()
            folder_path =self .folder_path_label .text ().replace ("Selected Folder: ","")

            if extension in [".dataset.npz",".MLdataset.npz"]:
                self .combine_npz_datasets (selected_files ,folder_path ,extension )
            elif extension in [".abf",".h5"]:
                self .combine_abf_h5_datasets (selected_files ,folder_path ,extension )
            elif extension ==".event_data.npz":
                self .combine_event_data_npz (selected_files ,folder_path )
            elif extension ==".event_fitting.npz":
                self .combine_event_fitting_npz (selected_files ,folder_path )
        else :
            QMessageBox .warning (self ,"Warning","No files selected.")


    def combine_event_fitting_npz (self ,selected_files ,folder_path ):
        combined_npz_dict ={}
        all_event_analyses =[]
        event_counter =0 

        file_lengths =[]
        if self .same_duration_checkbox .isChecked ():
            file_length ,ok =QInputDialog .getInt (self ,"File Duration","Enter the duration of each file in seconds:")
            if ok :
                file_lengths =[file_length ]*len (selected_files )
            else :
                QMessageBox .warning (self ,"Warning","File duration not provided. Skipping combination.")
                return 
        else :
            for file_path in selected_files :
                file_length ,ok =QInputDialog .getInt (self ,"File Duration",f"Enter the duration of file '{file_path }' in seconds:")
                if ok :
                    file_lengths .append (file_length )
                else :
                    QMessageBox .warning (self ,"Warning","File duration not provided. Skipping file.")
                    return 

        cumulative_time =0 
        for file_path ,file_length in zip (selected_files ,file_lengths ):
            full_path =os .path .join (folder_path ,file_path )
            with np .load (full_path ,allow_pickle =True )as data :
                for key ,value in data .items ():
                    if key .startswith ('EVENT_DATA_'):
                        event_num =int (key .split ('_')[2 ])
                        new_key =f'EVENT_DATA_{event_counter }'
                        if key .endswith ('_part_2'):
                            value =value +cumulative_time 
                        combined_npz_dict [f'{new_key }{key [key .rfind ("_"):]}']=value 
                    elif key .startswith ('SEGMENT_INFO_'):
                        event_num =int (key .split ('_')[2 ])
                        new_key =f'SEGMENT_INFO_{event_counter }'
                        combined_npz_dict [f'{new_key }{key [key .rfind ("_"):]}']=value 
                    elif key .startswith ('EVENT_ANALYSIS_'):
                        event_num =int (key .split ('_')[2 ])
                        new_key =f'EVENT_ANALYSIS_{event_counter }'
                        adjusted_value =value .copy ()
                        adjusted_value [-1 ]+=cumulative_time 
                        combined_npz_dict [new_key ]=adjusted_value 
                        all_event_analyses .append (adjusted_value )
                    event_counter +=1 
            cumulative_time +=file_length 

        if combined_npz_dict :
            save_path ,_ =QFileDialog .getSaveFileName (self ,"Save Combined Event Fitting Data","","NumPy Files (*.event_fitting.npz)")
            if save_path :
                if not save_path .endswith ('.event_fitting.npz'):
                    save_path +='.event_fitting.npz'
                np .savez_compressed (save_path ,**combined_npz_dict )


                analysis_save_path =save_path .replace ('.event_fitting.npz','.event_analysis.npz')
                np .savez_compressed (analysis_save_path ,event_analyses =np .array (all_event_analyses ,dtype =object ))

                QMessageBox .information (self ,"Success","Event fitting data files combined successfully.")
        else :
            QMessageBox .warning (self ,"Warning","No valid event fitting data files to combine.")

    def combine_event_data_npz (self ,selected_files ,folder_path ):
        combined_events =[]
        sampling_rate =None 
        event_id_offset =0 
        cumulative_time =0 

        file_lengths =[]
        if self .same_duration_checkbox .isChecked ():
            file_length ,ok =QInputDialog .getInt (self ,"File Duration","Enter the duration of each file in seconds:")
            if ok :
                file_lengths =[file_length ]*len (selected_files )
            else :
                QMessageBox .warning (self ,"Warning","File duration not provided. Skipping combination.")
                return 
        else :
            for file_path in selected_files :
                file_length ,ok =QInputDialog .getInt (self ,"File Duration",f"Enter the duration of file '{file_path }' in seconds:")
                if ok :
                    file_lengths .append (file_length )
                else :
                    QMessageBox .warning (self ,"Warning","File duration not provided. Skipping file.")
                    return 

        for file_path ,file_length in zip (selected_files ,file_lengths ):
            full_path =os .path .join (folder_path ,file_path )
            with np .load (full_path ,allow_pickle =True )as data :
                if sampling_rate is None :
                    sampling_rate =data ['sampling_rate']
                elif sampling_rate !=data ['sampling_rate']:
                    QMessageBox .warning (self ,"Warning",f"File {file_path } has a different sampling rate. Skipping.")
                    continue 

                events =data ['events']
                for event in events :
                    adjusted_event =event .copy ()
                    adjusted_event ['event_id']+=event_id_offset 
                    adjusted_event ['start_time']+=cumulative_time 
                    adjusted_event ['end_time']+=cumulative_time 
                    combined_events .append (adjusted_event )

            event_id_offset =len (combined_events )
            cumulative_time +=file_length 

        if combined_events :
            save_path ,_ =QFileDialog .getSaveFileName (self ,"Save Combined Event Data","","NumPy Files (*.event_data.npz)")
            if save_path :
                if not save_path .endswith ('.event_data.npz'):
                    save_path +='.event_data.npz'
                np .savez_compressed (save_path ,sampling_rate =sampling_rate ,events =combined_events )
                QMessageBox .information (self ,"Success","Event data files combined successfully.")
        else :
            QMessageBox .warning (self ,"Warning","No valid event data files to combine.")

    def combine_npz_datasets (self ,selected_files ,folder_path ,extension ):
        file_lengths =[]
        if self .same_duration_checkbox .isChecked ():
            file_length ,ok =QInputDialog .getInt (self ,"File Duration","Enter the duration of each file in seconds:")
            if ok :
                file_lengths =[file_length ]*len (selected_files )
            else :
                QMessageBox .warning (self ,"Warning","File duration not provided. Skipping combination.")
                return 
        else :
            for file_path in selected_files :
                file_length ,ok =QInputDialog .getInt (self ,"File Duration",f"Enter the duration of file '{file_path }' in seconds:")
                if ok :
                    file_lengths .append (file_length )
                else :
                    QMessageBox .warning (self ,"Warning","File duration not provided. Skipping file.")
                    return 

        combined_data =None 
        settings_file =None 
        cumulative_time =0 
        for file_path ,file_length in zip (selected_files ,file_lengths ):
            full_path =os .path .join (folder_path ,file_path )
            with np .load (full_path )as data :
                if combined_data is None :
                    combined_data =data ['X']
                else :
                    try :
                        data_x =data ['X']
                        data_x [:,8 ]+=cumulative_time 
                        combined_data =np .concatenate ((combined_data ,data_x ))
                    except :
                        print (f"Cannot save file: {file_path }")
                if settings_file is None :
                    settings_file =data ['settings']
            cumulative_time +=file_length 

        save_path ,_ =QFileDialog .getSaveFileName (self ,"Save Combined Dataset","",f"NumPy Files (*{extension })")
        if save_path :
            if not save_path .endswith (extension ):
                save_path +=extension 
            np .savez (save_path ,X =combined_data ,settings =settings_file )
            QMessageBox .information (self ,"Success","Datasets/Files combined successfully.")

    def combine_abf_h5_datasets (self ,selected_files ,folder_path ,extension ):
        combined_data =[]
        sampling_rate =None 

        for file_path in selected_files :
            full_path =os .path .join (folder_path ,file_path )
            if extension ==".abf":
                data ,file_sampling_rate =self .read_abf_file (full_path )
            elif extension ==".h5":
                with h5py .File (full_path ,'r')as f :
                    data =f ['selected_data'][:]
                    file_sampling_rate =f .attrs ['sampling_rate']

            combined_data .append (data )

            if sampling_rate is None :
                sampling_rate =file_sampling_rate 
            elif sampling_rate !=file_sampling_rate :
                QMessageBox .warning (self ,"Warning","Files have different sampling rates. Proceeding with combination, but please be cautious with the results.")


        combined_data =np .concatenate (combined_data )


        save_path ,_ =QFileDialog .getSaveFileName (self ,"Save Combined Dataset","","HDF5 Files (*.h5)")
        if save_path :
            if not save_path .endswith ('.h5'):
                save_path +='.h5'
            with h5py .File (save_path ,'w')as f :
                f .create_dataset ('selected_data',data =combined_data )
                f .attrs ['sampling_rate']=sampling_rate 
            QMessageBox .information (self ,"Success","Datasets combined successfully and saved as .h5 file.")

if __name__ =='__main__':
    app =QApplication (sys .argv )
    app .setStyle (QStyleFactory .create ('Fusion'))


    palette =QPalette ()
    palette .setColor (QPalette .ColorRole .Window ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .WindowText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Base ,QColor (25 ,25 ,25 ))
    palette .setColor (QPalette .ColorRole .AlternateBase ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .ToolTipBase ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .ToolTipText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Text ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Button ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .ButtonText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .BrightText ,Qt .GlobalColor .red )
    palette .setColor (QPalette .ColorRole .Link ,QColor (42 ,130 ,218 ))
    palette .setColor (QPalette .ColorRole .Highlight ,QColor (42 ,130 ,218 ))
    palette .setColor (QPalette .ColorRole .HighlightedText ,Qt .GlobalColor .black )
    app .setPalette (palette )

    window =SDCombineDatasets ()
    window .show ()
    sys .exit (app .exec ())




import os 
import sys 

from macholib .MachOStandalone import MachOStandalone 
from macholib .util import strip_files 


def standaloneApp (path ):
    if not (os .path .isdir (path )and os .path .exists (os .path .join (path ,"Contents"))):
        print ("%s: %s does not look like an app bundle"%(sys .argv [0 ],path ))
        sys .exit (1 )
    files =MachOStandalone (path ).run ()
    strip_files (files )


def main ():
    print (
    "WARNING: 'macho_standalone' is deprecated, use "
    "'python -mmacholib standalone' instead"
    )
    if not sys .argv [1 :]:
        raise SystemExit ("usage: %s [appbundle ...]"%(sys .argv [0 ],))
    for fn in sys .argv [1 :]:
        standaloneApp (fn )


if __name__ =="__main__":
    main ()



from .import PIL 
from .import automissing 
from .import autopackages 
from .import black 
from .import ctypes 
from .import detect_dunder_file 
from .import ftplib 
from .import gcloud 
from .import lxml 
from .import matplotlib 
from .import multiprocessing 
from .import opencv 
from .import pandas 
from .import platformdirs 
from .import pydantic 
from .import pydoc 
from .import pyenchant 
from .import pygame 
from .import pylsp 
from .import pyopengl 
from .import pyside 
from .import pyside2 
from .import pyside6 
from .import qt5 
from .import qt6 
from .import rtree 
from .import setuptools 
from .import shiboken2 
from .import shiboken6 
from .import sip 
from .import six 
from .import sphinx 
from .import sqlalchemy 
from .import sslmod 
from .import sysconfig_module 
from .import tkinter 
from .import virtualenv 
from .import wx 
from .import xml 
from .import zmq 




from __future__ import print_function 

from macholib ._cmdline import main as _main 


def print_file (fp ,path ):
    print (path ,file =fp )


def main ():
    print (
    "WARNING: 'macho_find' is deprecated, "  "use 'python -mmacholib dump' instead"
    )
    _main (print_file )


if __name__ =="__main__":
    try :
        main ()
    except KeyboardInterrupt :
        pass 



def _emulate_shell_environ ():
    import os 
    import sys 
    import time 

    if sys .version_info [0 ]>2 :
        env =os .environb 
    else :
        env =os .environ 

    split_char =b"="








    try :
        login =os .getlogin ()
        if login =="root":


            try :
                login =os .environ ["LOGNAME"]
            except KeyError :
                login =None 
    except AttributeError :
        try :
            login =os .environ ["LOGNAME"]
        except KeyError :
            login =None 

    if login is None :
        return 

    master ,slave =os .openpty ()
    pid =os .fork ()

    if pid ==0 :

        os .close (master )
        os .setsid ()
        os .dup2 (slave ,0 )
        os .dup2 (slave ,1 )
        os .dup2 (slave ,2 )
        os .execv ("/usr/bin/login",["login","-qf",login ])
        os ._exit (42 )

    else :

        os .close (slave )



        os .write (master ,b'echo "---------";env;echo "-----------"\r\n')
        os .write (master ,b"exit\r\n")
        time .sleep (1 )

        data =[]
        b =os .read (master ,2048 )
        while b :
            data .append (b )
            b =os .read (master ,2048 )
        data =b"".join (data )
        os .waitpid (pid ,0 )

    in_data =False 
    for ln in data .splitlines ():
        if not in_data :
            if ln .strip ().startswith (b"--------"):
                in_data =True 
            continue 

        if ln .startswith (b"--------"):
            break 

        try :
            key ,value =ln .rstrip ().split (split_char ,1 )
        except Exception :
            pass 

        else :
            env [key ]=value 


_emulate_shell_environ ()



import distutils .sysconfig 
import distutils .util 
import os 
import re 
import sys 

gPreBuildVariants =[
{
"name":"main-universal2",
"target":"10.9",
"cflags":"-g -arch arm64 -arch x86_64",
"cc":"/usr/bin/clang",
},
{
"name":"main-arm64",
"target":"10.16",
"cflags":"-g -arch arm64",
"cc":"/usr/bin/clang",
},
{
"name":"main-universal",
"target":"10.5",
"cflags":"-g -isysroot /Developer/SDKs/MacOSX10.5.sdk "
"-arch i386 -arch ppc -arch ppc64 -arch x86_64",
"cc":"gcc-4.2",
},
{
"name":"main-ppc64",
"target":"10.5",
"cflags":"-g -isysroot /Developer/SDKs/MacOSX10.5.sdk -arch ppc64",
"cc":"gcc-4.2",
},
{
"name":"main-x86_64",
"target":"10.5",
"cflags":"-g -arch x86_64 -Wl,-rpath,@executable_path/../Frameworks",
"cc":"/usr/bin/clang",
},
{
"name":"main-fat3",
"target":"10.5",
"cflags":"-g -isysroot / -arch i386 -arch ppc -arch x86_64",
"cc":"gcc-4.2",
},
{
"name":"main-intel",
"target":"10.5",
"cflags":"-g -arch i386 -arch x86_64 -fexceptions",
"cc":"/usr/bin/clang",
},
{
"name":"main-i386",
"target":"10.4",
"cflags":"-g -arch i386",
"cc":"/usr/bin/clang",
},
{
"name":"main-ppc",
"target":"10.3",
"cflags":"-g -isysroot /Developer/SDKs/MacOSX10.4u.sdk -arch ppc",
"cc":"gcc-4.0",
},
{
"name":"main-fat",
"target":"10.3",
"cflags":"-g -isysroot /Developer/SDKs/MacOSX10.4u.sdk "
"-arch i386 -arch ppc",
"cc":"gcc-4.0",
},
]


def main (all =False ,arch =None ,secondary =False ,redirect_asl =False ,use_old_sdk =False ):
    basepath =os .path .dirname (__file__ )
    builddir =os .path .join (basepath ,"prebuilt")
    if not os .path .exists (builddir ):
        os .makedirs (builddir )
    src =os .path .join (basepath ,"src","main.c")

    cfg =distutils .sysconfig .get_config_vars ()

    BASE_CFLAGS =cfg ["CFLAGS"]
    BASE_CFLAGS =BASE_CFLAGS .replace ("-dynamic","")
    while True :
        x =re .sub (r"-arch\s+\S+","",BASE_CFLAGS )
        if x ==BASE_CFLAGS :
            break 
        BASE_CFLAGS =x 

    while True :
        x =re .sub (r"-isysroot\s+\S+","",BASE_CFLAGS )
        if x ==BASE_CFLAGS :
            break 
        BASE_CFLAGS =x 

    if arch is None :
        arch =distutils .util .get_platform ().split ("-")[-1 ]
        if sys .prefix .startswith ("/System")and sys .version_info [:2 ]==(2 ,5 ):
            arch ="fat"

    name ="main-"+arch 
    root =None 

    if all :
        for entry in gPreBuildVariants :
            dest =os .path .join (builddir ,entry ["name"])

            for replace in (0 ,1 ):
                if replace :
                    dest =os .path .join (
                    builddir ,entry ["name"].replace ("main","secondary")
                    )

                if not os .path .exists (dest )or (
                os .stat (dest ).st_mtime <os .stat (src ).st_mtime 
                ):
                    if root is None :
                        fp =os .popen ("xcode-select -print-path","r")
                        root =fp .read ().strip ()
                        fp .close ()

                    print ("rebuilding %s"%(os .path .basename (dest ),))

                    CC =os .path .join (root ,"usr","bin",entry ["cc"])
                    CFLAGS =(
                    BASE_CFLAGS 
                    +" "
                    +entry ["cflags"].replace ("@@XCODE_ROOT@@",root )
                    )
                    if replace :
                        CFLAGS +=" -DPY2APP_SECONDARY"
                    os .environ ["MACOSX_DEPLOYMENT_TARGET"]=entry ["target"]
                    os .system (
                    (
                    '"%(CC)s" -o "%(dest)s" "%(src)s" %(CFLAGS)s '
                    "-framework Cocoa"
                    )
                    %locals ()
                    )

                    if not replace :
                        CFLAGS +=" -DREDIRECT_ASL"
                        dest =dest .replace ("main","main-asl")
                        os .system (
                        (
                        '"%(CC)s" -o "%(dest)s" "%(src)s" %(CFLAGS)s '
                        "-framework Cocoa"
                        )
                        %locals ()
                        )

    if secondary :
        name ="secondary-"
    elif redirect_asl :
        name ="main-asl-"
    else :
        name ="main-"

    dest =os .path .join (builddir ,name +arch )
    if not os .path .exists (dest )and name =="main-asl-":
        name ="main-"
        dest =os .path .join (builddir ,name +arch )

    if use_old_sdk :
        if os .path .exists (dest +"-oldsdk"):
            dest =dest +"-oldsdk"

    return dest 


if __name__ =="__main__":
    main (all =True )



def check (cmd ,mf ):



    m =mf .findNode ("wx.lib.pubsub")
    if m is None or m .filename is None :
        return None 

    include_packages =[
    "wx.lib.pubsub.*",
    "wx.lib.pubsub.core.*",
    "wx.lib.pubsub.core.arg1.*",
    "wx.lib.pubsub.core.kwargs.*",
    "wx.lib.pubsub.pubsub1.*",
    "wx.lib.pubsub.pubsub2.*",
    "wx.lib.pubsub.utils.*",
    ]
    return {"includes":include_packages }



from __future__ import absolute_import 

import io 
import os 
import sys 

PRESCRIPT ="""
def _setup_openssl():
    import os
    resourcepath = os.environ["RESOURCEPATH"]
    os.environ["%(openssl_cafile_env)s"] = os.path.join(
        resourcepath, "openssl.ca", "%(cafile_path)s")
    os.environ["%(openssl_capath_env)s"] = os.path.join(
        resourcepath, "openssl.ca", "%(capath_path)s")

_setup_openssl()
"""

if sys .version_info [:2 ]>=(3 ,4 ):

    def check (cmd ,mf ):
        m =mf .findNode ("ssl")
        if m is None or m .filename is None :
            return None 

        import ssl 

        datafiles =[]
        paths =ssl .get_default_verify_paths ()
        if paths .cafile is not None :
            datafiles .append (paths .cafile )
            cafile_path =os .path .basename (paths .cafile )
        else :
            cafile_path ="no-such-file"

        if paths .capath is not None :
            datafiles .append (paths .capath )
            capath_path =os .path .basename (paths .capath )
        else :
            capath_path ="no-such-file"

        prescript =PRESCRIPT %{
        "openssl_cafile_env":paths .openssl_cafile_env ,
        "openssl_capath_env":paths .openssl_capath_env ,
        "cafile_path":cafile_path ,
        "capath_path":capath_path ,
        }

        return {
        "resources":[("openssl.ca",datafiles )],
        "prescripts":[io .StringIO (prescript )],
        }



import os 
from collections import deque 

from macholib .dyld import framework_info 
from macholib .MachOGraph import MachOGraph ,MissingMachO 
from macholib .util import (
flipwritable ,
has_filename_filter ,
in_system_path ,
iter_platform_files ,
mergecopy ,
mergetree ,
)


class ExcludedMachO (MissingMachO ):
    pass 


class FilteredMachOGraph (MachOGraph ):
    def __init__ (self ,delegate ,*args ,**kwargs ):
        super (FilteredMachOGraph ,self ).__init__ (*args ,**kwargs )
        self .delegate =delegate 

    def createNode (self ,cls ,name ):
        cls =self .delegate .getClass (name ,cls )
        res =super (FilteredMachOGraph ,self ).createNode (cls ,name )
        return self .delegate .update_node (res )

    def locate (self ,filename ,loader =None ):
        newname =super (FilteredMachOGraph ,self ).locate (filename ,loader )
        if newname is None :
            return None 
        return self .delegate .locate (newname ,loader =loader )


class MachOStandalone (object ):
    def __init__ (self ,base ,dest =None ,graph =None ,env =None ,executable_path =None ):
        self .base =os .path .join (os .path .abspath (base ),"")
        if dest is None :
            dest =os .path .join (self .base ,"Contents","Frameworks")
        self .dest =dest 
        self .mm =FilteredMachOGraph (
        self ,graph =graph ,env =env ,executable_path =executable_path 
        )
        self .changemap ={}
        self .excludes =[]
        self .pending =deque ()

    def update_node (self ,m ):
        return m 

    def getClass (self ,name ,cls ):
        if in_system_path (name ):
            return ExcludedMachO 
        for base in self .excludes :
            if name .startswith (base ):
                return ExcludedMachO 
        return cls 

    def locate (self ,filename ,loader =None ):
        if in_system_path (filename ):
            return filename 
        if filename .startswith (self .base ):
            return filename 
        for base in self .excludes :
            if filename .startswith (base ):
                return filename 
        if filename in self .changemap :
            return self .changemap [filename ]
        info =framework_info (filename )
        if info is None :
            res =self .copy_dylib (filename )
            self .changemap [filename ]=res 
            return res 
        else :
            res =self .copy_framework (info )
            self .changemap [filename ]=res 
            return res 

    def copy_dylib (self ,filename ):




        if os .path .islink (filename ):
            dest =os .path .join (self .dest ,os .path .basename (os .path .realpath (filename )))
        else :
            dest =os .path .join (self .dest ,os .path .basename (filename ))

        if not os .path .exists (dest ):
            self .mergecopy (filename ,dest )
        return dest 

    def mergecopy (self ,src ,dest ):
        return mergecopy (src ,dest )

    def mergetree (self ,src ,dest ):
        return mergetree (src ,dest )

    def copy_framework (self ,info ):
        dest =os .path .join (self .dest ,info ["shortname"]+".framework")
        destfn =os .path .join (self .dest ,info ["name"])
        src =os .path .join (info ["location"],info ["shortname"]+".framework")
        if not os .path .exists (dest ):
            self .mergetree (src ,dest )
            self .pending .append ((destfn ,iter_platform_files (dest )))
        return destfn 

    def run (self ,platfiles =None ,contents =None ):
        mm =self .mm 
        if contents is None :
            contents ="@executable_path/.."
        if platfiles is None :
            platfiles =iter_platform_files (self .base )

        for fn in platfiles :
            mm .run_file (fn )

        while self .pending :
            fmwk ,files =self .pending .popleft ()
            ref =mm .findNode (fmwk )
            for fn in files :
                mm .run_file (fn ,caller =ref )

        changemap ={}
        skipcontents =os .path .join (os .path .dirname (self .dest ),"")
        machfiles =[]

        for node in mm .flatten (has_filename_filter ):
            machfiles .append (node )
            dest =os .path .join (
            contents ,
            os .path .normpath (node .filename [len (skipcontents ):]),
            )
            changemap [node .filename ]=dest 

        def changefunc (path ):
            print ("changefunc: ",path )
            if path .startswith ("@loader_path/"):




                return path 

            elif path .startswith ("@rpath/"):


                return path 

            res =mm .locate (path )
            rv =changemap .get (res )
            if rv is None and path .startswith ("@loader_path/"):
                rv =changemap .get (mm .locate (mm .trans_table .get ((node .filename ,path ))))
            if rv is None and path .startswith ("@rpath/"):
                rv =changemap .get (mm .locate (mm .trans_table .get ((node .filename ,path ))))
            return rv 

        for node in machfiles :
            fn =mm .locate (node .filename )
            if fn is None :
                continue 
            rewroteAny =False 
            for _header in node .headers :
                if node .rewriteLoadCommands (changefunc ):
                    rewroteAny =True 
            if rewroteAny :
                old_mode =flipwritable (fn )
                try :
                    with open (fn ,"rb+")as f :
                        for _header in node .headers :
                            f .seek (0 )
                            node .write (f )
                        f .seek (0 ,2 )
                        f .flush ()
                finally :
                    flipwritable (fn ,old_mode )

        allfiles =[mm .locate (node .filename )for node in machfiles ]
        return set (filter (None ,allfiles ))







import os 
import sys 
import textwrap 

try :
    from cStringIO import StringIO 
except ImportError :
    from io import StringIO 



NEW_TK =(8 ,6 ,11 )


def tk_version ():

    import _tkinter 

    tk =_tkinter .create ()
    version_string =tk .call ("info","patchlevel")
    return tuple (int (x )for x in version_string .split ("."))


def check (cmd ,mf ):
    m =mf .findNode ("_tkinter")
    if m is None :
        return None 

    try :
        import _tkinter 
    except ImportError :
        return None 

    prefix =sys .prefix if not hasattr (sys ,"real_prefix")else sys .real_prefix 

    paths =[]
    lib =os .path .join (prefix ,"lib")
    for fn in os .listdir (lib ):
        if not os .path .isdir (os .path .join (lib ,fn )):
            continue 

        if fn .startswith ("tk"):
            tk_path =fn 
            paths .append (os .path .join (lib ,fn ))

        elif fn .startswith ("tcl"):
            tcl_path =fn 
            paths .append (os .path .join (lib ,fn ))

    if not paths :
        if tk_version ()<NEW_TK :
            return {"use_old_sdk":True }
        return None 

    prescript =(
    textwrap .dedent (
    """\
        def _boot_tkinter():
            import os

            resourcepath = os.environ["RESOURCEPATH"]
            os.putenv("TCL_LIBRARY", os.path.join(resourcepath, "lib/%(tcl_path)s"))
            os.putenv("TK_LIBRARY", os.path.join(resourcepath, "lib/%(tk_path)s"))
        _boot_tkinter()
        """
    )
    %{"tcl_path":tcl_path ,"tk_path":tk_path }
    )

    return {
    "resources":[("lib",paths )],
    "prescripts":[StringIO (prescript )],
    "use_old_sdk":tk_version ()<NEW_TK ,
    }



import sys 
from PySide6 .QtWidgets import (QApplication ,QWidget ,QVBoxLayout ,QHBoxLayout ,
QPushButton ,QGroupBox ,QLabel ,QFileDialog ,
QScrollArea ,QMainWindow ,QSizePolicy ,QSplitter ,
QRadioButton ,QVBoxLayout ,QLineEdit ,QCheckBox ,QDoubleSpinBox ,QTabWidget ,
QTableWidget ,QTableWidgetItem ,QSpinBox ,QStyleFactory ,QProgressDialog ,QMessageBox ,QInputDialog )
from PySide6 .QtCore import Qt ,QTimer ,QSize 
from PySide6 .QtGui import QFont ,QPalette ,QColor 
import numpy as np 
from matplotlib .backends .backend_qt5agg import FigureCanvasQTAgg as FigureCanvas 
from matplotlib .backends .backend_qt5agg import NavigationToolbar2QT 
from matplotlib .figure import Figure 
import matplotlib .pyplot as plt 
import csv 
import os 

class MplCanvas (FigureCanvas ):
    def __init__ (self ,parent =None ,width =5 ,height =4 ,dpi =100 ):
        fig =Figure (figsize =(width ,height ),dpi =dpi )
        self .axes =fig .add_subplot (111 )
        super (MplCanvas ,self ).__init__ (fig )

class ScatterCanvas (FigureCanvas ):
    def __init__ (self ,parent =None ,width =5 ,height =4 ,dpi =100 ):
        fig =Figure (figsize =(width ,height ),dpi =dpi )
        self .axes =fig .add_subplot (111 )
        super (ScatterCanvas ,self ).__init__ (fig )

class SDAnalysisApp (QMainWindow ):
    def __init__ (self ):
        super ().__init__ ()
        self .setWindowTitle ('SD Event Analysis App')
        self .setGeometry (100 ,100 ,1200 ,800 )

        self .central_widget =QWidget ()
        self .setCentralWidget (self .central_widget )
        self .main_splitter =QSplitter (Qt .Orientation .Horizontal )


        self .setup_left_panel ()


        self .setup_right_panel ()


        self .central_layout =QHBoxLayout (self .central_widget )
        self .central_layout .addWidget (self .main_splitter )

        self .data =None 
        self .events_data ={}
        self .classification_to_event_ids ={}
        self .selected_event_ids =set ()


        self .save_analysis_button =QPushButton ("Save Analysis")
        self .save_analysis_button .clicked .connect (self .save_analysis )
        self .top_group_layout .addWidget (self .save_analysis_button )



    def setup_left_panel (self ):
        self .top_group =QGroupBox ()
        self .top_group_layout =QVBoxLayout (self .top_group )


        self .configure_top_group ()


        self .event_categories_group_box ,self .event_categories_layout =self .setup_scroll_area ("Event Categories")
        self .event_classification_group_box ,self .event_classification_layout =self .setup_scroll_area ("Event Classifications")


        self .select_all_button =QPushButton ("Select All")
        self .select_all_button .clicked .connect (self .select_all_classifications )


        self .left_splitter =QSplitter (Qt .Orientation .Vertical )
        self .left_splitter .addWidget (self .top_group )
        self .left_splitter .addWidget (self .event_categories_group_box )
        self .left_splitter .addWidget (self .event_classification_group_box )
        self .left_splitter .addWidget (self .select_all_button )
        self .left_splitter .setSizes ([10 ,300 ,300 ,50 ])
        self .main_splitter .addWidget (self .left_splitter )

    def configure_top_group (self ):
        self .title_label =QLabel ('SD Event Analysis App')
        self .title_label .setFont (QFont ('Arial',23 ,QFont .Weight .Bold ))
        self .title_label .setAlignment (Qt .AlignmentFlag .AlignCenter )
        self .subtitle_label =QLabel ('shankar.dutt@anu.edu.au')
        self .subtitle_label .setFont (QFont ('Arial',15 ,QFont .Weight .Bold ))
        self .subtitle_label .setAlignment (Qt .AlignmentFlag .AlignCenter )
        self .top_group_layout .addWidget (self .title_label )
        self .top_group_layout .addWidget (self .subtitle_label )

        self .file_button =QPushButton ('Select File')
        self .file_button .clicked .connect (self .load_file )
        self .top_group_layout .addWidget (self .file_button )


        self .threshold_container =QWidget ()
        self .threshold_container_layout =QHBoxLayout (self .threshold_container )

        self .threshold_label =QLabel ("Similarity Threshold: ")
        self .threshold_container_layout .addWidget (self .threshold_label )

        self .threshold_input =QDoubleSpinBox ()
        self .threshold_input .setSuffix (" %")
        self .threshold_input .setRange (0 ,100 )
        self .threshold_input .setValue (90 )
        self .threshold_input .setSingleStep (1 )


        self .threshold_timer =QTimer (self )
        self .threshold_timer .setSingleShot (True )
        self .threshold_timer .timeout .connect (self .on_threshold_changed )
        self .threshold_input .valueChanged .connect (self .start_threshold_timer )

        self .threshold_container_layout .addWidget (self .threshold_input )

        self .reclassify_checkbox =QCheckBox ("Reclassify the event categories based on threshold")
        self .reclassify_checkbox .stateChanged .connect (self .on_threshold_changed )
        self .top_group_layout .addWidget (self .threshold_container )
        self .top_group_layout .addWidget (self .reclassify_checkbox )

    def start_threshold_timer (self ):
        self .threshold_timer .start (500 )

    def setup_right_panel (self ):
        self .right_panel =QWidget ()
        self .right_splitter =QSplitter (Qt .Orientation .Vertical )


        self .tabs =QTabWidget ()
        self .histograms_tab =QWidget ()
        self .scatter_plots_tab =QWidget ()
        self .tabs .addTab (self .histograms_tab ,"Histograms")
        self .tabs .addTab (self .scatter_plots_tab ,"Scatter Plots")


        self .setup_histograms_tab ()


        self .setup_scatter_plots_tab ()

        self .right_splitter .addWidget (self .tabs )


        self .histograms_group =QGroupBox ("Histograms")
        self .histograms_horizontal_layout =QHBoxLayout (self .histograms_group )



        self .bottom_right_splitter =QSplitter (Qt .Orientation .Horizontal )


        self .event_plots_group =QGroupBox ("Event Plots")
        self .event_plots_layout =QVBoxLayout (self .event_plots_group )
        self .event_plot_canvas =FigureCanvas (Figure (figsize =(5 ,3 )))
        self .event_plots_layout .addWidget (self .event_plot_canvas )
        self .event_plot_toolbar =NavigationToolbar2QT (self .event_plot_canvas ,self .event_plots_group )
        self .event_plot_toolbar .setIconSize (QSize (16 ,16 ))
        self .event_plots_layout .addWidget (self .event_plot_toolbar )


        self .event_navigation_layout =QHBoxLayout ()
        self .prev_button =QPushButton ("Previous")
        self .event_navigation_layout .addWidget (self .prev_button )

        self .next_button =QPushButton ("Next")
        self .event_navigation_layout .addWidget (self .next_button )
        self .event_plots_layout .addLayout (self .event_navigation_layout )
        self .next_button .clicked .connect (self .next_event )
        self .prev_button .clicked .connect (self .previous_event )


        self .bottom_right_splitter .addWidget (self .event_plots_group )


        self .table_group =QGroupBox ("Event Information")
        self .table_layout =QVBoxLayout (self .table_group )
        self .event_info_table =QTableWidget (10 ,3 )
        self .event_info_table .setHorizontalHeaderLabels (['Type','Value','Description'])
        self .table_layout .addWidget (self .event_info_table )
        self .bottom_right_splitter .addWidget (self .table_group )

        self .right_splitter .addWidget (self .bottom_right_splitter )
        self .main_splitter .addWidget (self .right_splitter )


        self .main_splitter .setSizes ([300 ,800 ])
        self .right_splitter .setSizes ([400 ,350 ])
        self .bottom_right_splitter .setSizes ([400 ,400 ])

    def setup_histograms_tab (self ):
        self .histograms_layout =QVBoxLayout (self .histograms_tab )
        self .histograms_group =QGroupBox ("Histograms")
        self .histograms_horizontal_layout =QHBoxLayout (self .histograms_group )


        self .all_events_layout =QVBoxLayout ()
        self .all_events_histogram_canvas =FigureCanvas (Figure (figsize =(5 ,3 )))
        self .all_events_layout .addWidget (self .all_events_histogram_canvas )
        self .all_events_histogram_toolbar =NavigationToolbar2QT (self .all_events_histogram_canvas ,self .histograms_group )
        self .all_events_histogram_toolbar .setIconSize (QSize (16 ,16 ))
        self .all_events_layout .addWidget (self .all_events_histogram_toolbar )
        self .histograms_horizontal_layout .addLayout (self .all_events_layout )


        self .selected_classifications_layout =QVBoxLayout ()
        self .selected_classifications_histogram_canvas =FigureCanvas (Figure (figsize =(5 ,3 )))
        self .selected_classifications_layout .addWidget (self .selected_classifications_histogram_canvas )
        self .selected_classifications_histogram_toolbar =NavigationToolbar2QT (self .selected_classifications_histogram_canvas ,self .histograms_group )
        self .selected_classifications_histogram_toolbar .setIconSize (QSize (16 ,16 ))
        self .selected_classifications_layout .addWidget (self .selected_classifications_histogram_toolbar )
        self .histograms_horizontal_layout .addLayout (self .selected_classifications_layout )

        self .histograms_layout .addWidget (self .histograms_group )

    def setup_scatter_plots_tab (self ):
        self .scatter_plots_layout =QVBoxLayout (self .scatter_plots_tab )
        self .scatter_plots_group =QGroupBox ("Scatter Plots")
        self .scatter_plots_horizontal_layout =QHBoxLayout (self .scatter_plots_group )


        self .all_events_scatter_layout =QVBoxLayout ()
        self .all_events_scatter_canvas =ScatterCanvas (self ,width =5 ,height =3 )
        self .all_events_scatter_layout .addWidget (self .all_events_scatter_canvas )
        self .all_events_scatter_toolbar =NavigationToolbar2QT (self .all_events_scatter_canvas ,self .scatter_plots_group )
        self .all_events_scatter_toolbar .setIconSize (QSize (16 ,16 ))
        self .all_events_scatter_layout .addWidget (self .all_events_scatter_toolbar )
        self .scatter_plots_horizontal_layout .addLayout (self .all_events_scatter_layout )


        self .selected_classifications_scatter_layout =QVBoxLayout ()
        self .selected_classifications_scatter_canvas =ScatterCanvas (self ,width =5 ,height =3 )
        self .selected_classifications_scatter_layout .addWidget (self .selected_classifications_scatter_canvas )
        self .selected_classifications_scatter_toolbar =NavigationToolbar2QT (self .selected_classifications_scatter_canvas ,self .scatter_plots_group )
        self .selected_classifications_scatter_toolbar .setIconSize (QSize (16 ,16 ))
        self .selected_classifications_scatter_layout .addWidget (self .selected_classifications_scatter_toolbar )
        self .scatter_plots_horizontal_layout .addLayout (self .selected_classifications_scatter_layout )

        self .scatter_plots_layout .addWidget (self .scatter_plots_group )


    def setup_scroll_area (self ,title ):
        scroll_area =QScrollArea ()
        scroll_area .setWidgetResizable (True )

        container_widget =QWidget ()
        layout =QVBoxLayout (container_widget )

        scroll_area .setWidget (container_widget )

        group_box =QGroupBox (title )
        group_layout =QVBoxLayout (group_box )
        group_layout .addWidget (scroll_area )

        return group_box ,layout 

    def on_threshold_changed (self ):
        if self .data is not None :
            self .prepare_and_display_event_data ()

            self .plot_all_events_histogram ()
            self .plot_all_events_scatter ()
            if self .selected_event_ids :
                self .plot_selected_events_histogram ()
                self .plot_selected_events_scatter ()

            if hasattr (self ,'current_event_index')and self .selected_event_ids :
                event_id =sorted (self .selected_event_ids )[self .current_event_index ]
                self .plot_event_data (event_id )
                self .display_segment_info (event_id )


            self .clear_layout (self .event_categories_layout )
            for category in sorted (self .events_data ,key =lambda x :int (x .split ()[0 ])):
                events_data =self .events_data [category ]
                radio_button =QRadioButton (f"{category } ({len (events_data )} events)")
                radio_button .category =category 
                radio_button .toggled .connect (self .on_radio_button_toggled )
                self .event_categories_layout .addWidget (radio_button )


            if self .event_categories_layout .count ()>0 :
                self .event_categories_layout .itemAt (0 ).widget ().setChecked (True )


    def load_file (self ):
        file_name ,_ =QFileDialog .getOpenFileName (self ,'Open NPZ File','','NPZ Files (*event_fitting.npz)')
        if file_name :
            self .data =np .load (file_name ,allow_pickle =True )
            self .prepare_and_display_event_data ()

    def prepare_and_display_event_data (self ):
        if self .data is not None :
            self .events_data .clear ()
            self .clear_layout (self .event_categories_layout )
            self .clear_layout (self .event_classification_layout )
            self .plot_all_events_histogram ()

            for key in self .data .files :
                if 'SEGMENT_INFO'in key and 'number_of_segments'in key :
                    event_id =int (key .split ('_')[2 ])
                    mean_diffs_key =f'SEGMENT_INFO_{event_id }_segment_mean_diffs'
                    segment_widths_key =f'SEGMENT_INFO_{event_id }_segment_widths_time'
                    if mean_diffs_key in self .data and segment_widths_key in self .data :
                        mean_diffs =self .data [mean_diffs_key ]
                        segment_widths =self .data [segment_widths_key ]
                        original_classification ,new_classification ,final_classification ,new_segment_count ,original_segment_count ,merged_segments =self .classify_event (mean_diffs )


                        category =f"{new_segment_count if self .reclassify_checkbox .isChecked ()else original_segment_count } segments"

                        self .events_data .setdefault (category ,[]).append ((event_id ,mean_diffs ,segment_widths ,
                        original_classification ,new_classification ,
                        final_classification ,new_segment_count ,
                        original_segment_count ,merged_segments ))
                    else :
                        original_segment_count =int (self .data [key ][0 ])
                        category =f"{original_segment_count } segments"
                        self .events_data .setdefault (category ,[]).append ((event_id ,[],[],'','','',
                        original_segment_count ,original_segment_count ,[]))

            for category in sorted (self .events_data ,key =lambda x :int (x .split ()[0 ])):
                events_data =self .events_data [category ]
                radio_button =QRadioButton (f"{category } ({len (events_data )} events)")
                radio_button .category =category 
                radio_button .toggled .connect (self .on_radio_button_toggled )
                self .event_categories_layout .addWidget (radio_button )

            if self .event_categories_layout .count ()>0 :
                self .event_categories_layout .itemAt (0 ).widget ().setChecked (True )

    def on_radio_button_toggled (self ):
        radio_button =self .sender ()
        if radio_button .isChecked ():
            self .update_classification_group (radio_button .category )

    def update_classification_group (self ,category ):
        self .clear_layout (self .event_classification_layout )
        classification_counts ={}
        self .classification_checkboxes =[]
        self .classification_to_event_ids .clear ()

        for event_data in self .events_data .get (category ,[]):
            event_id ,_ ,_ ,original_classification ,new_classification ,final_classification ,new_segment_count ,_ ,merged_segments =event_data 

            if self .reclassify_checkbox .isChecked ()and original_classification !=final_classification :
                display_classification =f"{original_classification } -> {final_classification }"
                classification_key =f"{original_classification } -> {final_classification }"
            else :
                display_classification =original_classification 
                classification_key =original_classification 


            display_string =""
            for i ,char in enumerate (final_classification if self .reclassify_checkbox .isChecked ()else original_classification ):
                if self .reclassify_checkbox .isChecked ()and merged_segments [i ]:
                    display_string +=f"<u>{char }</u>"
                else :
                    display_string +=char 

            if self .reclassify_checkbox .isChecked ()and original_classification !=final_classification :
                display_string =f"{original_classification } -> {display_string }"

            classification_counts [classification_key ]=classification_counts .get (classification_key ,0 )+1 

            if classification_key not in self .classification_to_event_ids :
                self .classification_to_event_ids [classification_key ]=[]
            self .classification_to_event_ids [classification_key ].append (event_id )

        for classification_key ,count in sorted (classification_counts .items (),key =lambda x :x [0 ]):
            checkbox =QCheckBox (f"Category {classification_key } ({count } events)")
            checkbox .classification =classification_key 
            checkbox .stateChanged .connect (self .on_checkbox_state_changed )
            self .event_classification_layout .addWidget (checkbox )
            self .classification_checkboxes .append (checkbox )


        self .select_all_button .clicked .disconnect ()
        self .select_all_button .clicked .connect (self .select_all_classifications )

    def classify_event (self ,mean_diffs ):
        original_segment_count =len (mean_diffs )
        merged_segments =[False ]*original_segment_count 

        def get_label (index ):
            return str (index +1 )if index <9 else chr (ord ('A')+index -9 )


        sorted_indices =np .argsort (np .abs (mean_diffs ))
        original_classifications =['']*original_segment_count 
        for i ,idx in enumerate (sorted_indices ):
            original_classifications [idx ]=get_label (i )

        if self .reclassify_checkbox .isChecked ():
            threshold_ratio =self .threshold_input .value ()/100.0 


            new_classifications =[get_label (0 )]
            merged_mean_diffs =[mean_diffs [0 ]]

            for i in range (1 ,original_segment_count ):
                prev_mean_diff =merged_mean_diffs [-1 ]
                current_mean_diff =mean_diffs [i ]
                lower_bound =prev_mean_diff -abs (prev_mean_diff )*(1 -threshold_ratio )
                upper_bound =prev_mean_diff +abs (prev_mean_diff )*(1 -threshold_ratio )

                if lower_bound <=current_mean_diff <=upper_bound :
                    new_classifications .append (new_classifications [-1 ])
                    merged_segments [i ]=True 

                    merged_mean_diffs [-1 ]=(merged_mean_diffs [-1 ]*new_classifications .count (new_classifications [-1 ])+current_mean_diff )/(new_classifications .count (new_classifications [-1 ])+1 )
                else :
                    new_classifications .append (get_label (len (set (new_classifications ))))
                    merged_mean_diffs .append (current_mean_diff )


            unique_classifications =sorted (set (new_classifications ))
            sorted_merged_diffs =sorted (set (zip (unique_classifications ,merged_mean_diffs )),key =lambda x :x [1 ])
            new_classification_to_final ={class_ :get_label (i )for i ,(class_ ,_ )in enumerate (sorted_merged_diffs )}

            final_classifications =[new_classification_to_final [c ]for c in new_classifications ]
            new_segment_count =len (set (final_classifications ))
        else :
            new_classifications =original_classifications 
            final_classifications =original_classifications 
            new_segment_count =original_segment_count 
            merged_segments =[False ]*original_segment_count 


        original_classification_string =''.join (original_classifications )
        new_classification_string =''.join (new_classifications )
        final_classification_string =''.join (final_classifications )

        return (original_classification_string ,new_classification_string ,
        final_classification_string ,new_segment_count ,
        original_segment_count ,merged_segments )

    def plot_selected_events_scatter (self ):
        all_mean_diffs =[]
        all_segment_widths =[]
        for event_id in self .selected_event_ids :
            for segment_count ,events in self .events_data .items ():
                for event_data in events :
                    if event_data [0 ]==event_id :
                        _ ,mean_diffs ,segment_widths ,*_ =event_data 
                        all_mean_diffs .extend (mean_diffs )
                        all_segment_widths .extend (segment_widths )
                        break 

        if len (all_mean_diffs )>0 and len (all_segment_widths )>0 :

            self .selected_classifications_scatter_canvas .figure .clear ()

            ax =self .selected_classifications_scatter_canvas .figure .subplots ()
            ax .scatter (np .log (np .array (all_segment_widths )*1e3 ),all_mean_diffs )
            ax .set_title ('Segment Mean Diffs vs log(dt (ms)) for Selected Events')
            ax .set_xlabel ('log(Δt (ms))')
            ax .set_ylabel ('Mean Diff')
            self .selected_classifications_scatter_canvas .figure .tight_layout ()
            self .selected_classifications_scatter_canvas .draw ()


    def clear_layout (self ,layout ):
        while layout .count ():
            child =layout .takeAt (0 )
            if child .widget ():
                child .widget ().deleteLater ()

    def select_all_classifications (self ):
        try :
            if any (not checkbox .isChecked ()for checkbox in self .classification_checkboxes ):
                for checkbox in self .classification_checkboxes :
                    checkbox .setChecked (True )
            else :
                for checkbox in self .classification_checkboxes :
                    checkbox .setChecked (False )


            self .select_all_button .setText ("Unselect All"if any (checkbox .isChecked ()for checkbox in self .classification_checkboxes )else "Select All")
        except :
            pass 


    def plot_all_events_histogram (self ):
        max_mean_diffs =[]
        for key in self .data .files :
            if 'segment_mean_diffs'in key :
                mean_diffs =self .data [key ]
                max_mean_diffs .append (np .max (mean_diffs ))


        self .all_events_histogram_canvas .figure .clear ()

        ax =self .all_events_histogram_canvas .figure .subplots ()


        _ ,bins_auto =np .histogram (max_mean_diffs ,bins ='auto')
        num_bins_auto =len (bins_auto )-1 
        doubled_num_bins =num_bins_auto *2 


        new_bins =np .linspace (bins_auto [0 ],bins_auto [-1 ],doubled_num_bins +1 )

        ax .hist (max_mean_diffs ,bins =new_bins )
        ax .set_title ('Max Segment Mean Diffs for All Events')
        ax .set_xlabel ('Max Mean Diff')
        ax .set_ylabel ('Frequency')
        self .all_events_histogram_canvas .figure .tight_layout ()
        self .all_events_histogram_canvas .draw ()

        self .plot_all_events_scatter ()

    def on_checkbox_state_changed (self ):
        self .selected_event_ids .clear ()

        for checkbox in self .classification_checkboxes :
            if checkbox .isChecked ():

                event_ids =self .classification_to_event_ids .get (checkbox .classification ,[])
                self .selected_event_ids .update (event_ids )


        self .current_event_index =0 
        if self .selected_event_ids :
            first_event_id =next (iter (sorted (self .selected_event_ids )))
            self .plot_event_data (first_event_id )
            self .display_segment_info (first_event_id )


        self .plot_selected_events_histogram ()
        self .plot_selected_events_scatter ()

    def setup_navigation_buttons (self ):
        self .current_event_index =0 
        self .previous_button .clicked .connect (self .previous_event )
        self .next_button .clicked .connect (self .next_event )

    def previous_event (self ):
        if self .selected_event_ids and self .current_event_index >0 :
            self .current_event_index -=1 
            event_id =sorted (self .selected_event_ids )[self .current_event_index ]
            self .plot_event_data (event_id )
            self .display_segment_info (event_id )

    def next_event (self ):
        if self .selected_event_ids and self .current_event_index <len (self .selected_event_ids )-1 :
            self .current_event_index +=1 
            event_id =sorted (self .selected_event_ids )[self .current_event_index ]
            self .plot_event_data (event_id )
            self .display_segment_info (event_id )

    def plot_selected_events_histogram (self ):
        all_mean_diffs =[]
        for event_id in self .selected_event_ids :
            key =f'SEGMENT_INFO_{event_id }_segment_mean_diffs'
            if key in self .data :
                mean_diffs =self .data [key ]
                all_mean_diffs .extend (mean_diffs )

        if len (all_mean_diffs )>1 :

            self .selected_classifications_histogram_canvas .figure .clear ()

            ax =self .selected_classifications_histogram_canvas .figure .subplots ()


            num_bins ='auto'


            ax .hist (all_mean_diffs ,bins =num_bins ,edgecolor ='black')

            ax .set_title ('Segment Mean Diffs for Selected Events')
            ax .set_xlabel ('Mean Diff')
            ax .set_ylabel ('Frequency')
            self .selected_classifications_histogram_canvas .figure .tight_layout ()
            self .selected_classifications_histogram_canvas .draw ()


        self .plot_selected_events_scatter ()

    def plot_all_events_scatter (self ):
        max_mean_diffs =[]
        event_widths =[]
        for key in self .data .files :
            if 'segment_mean_diffs'in key :
                mean_diffs =self .data [key ]
                max_mean_diffs .append (np .max (mean_diffs ))

            if 'event_width'in key :
                event_widths .append (self .data [key ])

        if len (max_mean_diffs )>0 and len (event_widths )>0 :

            self .all_events_scatter_canvas .figure .clear ()

            ax =self .all_events_scatter_canvas .figure .subplots ()
            ax .scatter (np .log (np .array (event_widths )*1e3 ),max_mean_diffs )
            ax .set_title ('Max Segment Mean Diffs vs log(Event Width) for All Events')
            ax .set_xlabel ('log(Δt (ms))')
            ax .set_ylabel ('ΔI')
            self .all_events_scatter_canvas .figure .tight_layout ()
            self .all_events_scatter_canvas .draw ()

    def plot_event_data (self ,event_id ):

        x_values =self .data [f'EVENT_DATA_{event_id }_part_0']
        y_values_event =self .data [f'EVENT_DATA_{event_id }_part_1']
        y_values_fit =self .data [f'EVENT_DATA_{event_id }_part_3']


        self .event_plot_canvas .figure .clear ()


        ax =self .event_plot_canvas .figure .subplots ()
        ax .plot (x_values ,y_values_event ,label ='Event Data')
        ax .plot (x_values ,y_values_fit ,label ='Fit Data',linestyle ='--')


        ax .legend ()
        ax .set_title (f'Event {event_id } Data')
        ax .set_xlabel ('Time')
        ax .set_ylabel ('Data')

        self .event_plot_canvas .figure .tight_layout ()

        self .event_plot_canvas .draw ()



    def display_segment_info (self ,event_id ):
        for category ,events in self .events_data .items ():
            for event in events :
                if event [0 ]==event_id :
                    original_segment_count =event [7 ]
                    segment_mean_diffs =event [1 ]
                    segment_widths_time =event [2 ]
                    merged_segments =event [8 ]
                    break 
            else :
                continue 
            break 

        self .event_info_table .setRowCount (original_segment_count )
        self .event_info_table .setColumnCount (3 )
        self .event_info_table .setHorizontalHeaderLabels (['Segment','Mean Diff','Width Time'])

        for i in range (original_segment_count ):
            segment_item =QTableWidgetItem (f"{i +1 }")
            mean_diff_item =QTableWidgetItem (f"{segment_mean_diffs [i ]:.3g}")
            width_time_item =QTableWidgetItem (f"{segment_widths_time [i ]:.3g}")

            if self .reclassify_checkbox .isChecked ()and merged_segments [i ]:
                font =segment_item .font ()
                font .setUnderline (True )
                segment_item .setFont (font )
                mean_diff_item .setFont (font )
                width_time_item .setFont (font )

            self .event_info_table .setItem (i ,0 ,segment_item )
            self .event_info_table .setItem (i ,1 ,mean_diff_item )
            self .event_info_table .setItem (i ,2 ,width_time_item )

        self .event_info_table .resizeColumnsToContents ()
        self .event_info_table .resizeRowsToContents ()


    def save_analysis (self ):
        if not self .data :
            QMessageBox .warning (self ,"No Data","Please load data before saving analysis.")
            return 

        try :

            save_dir =QFileDialog .getExistingDirectory (self ,"Select Directory to Save Analysis")
            if not save_dir :
                return 


            csv_name ,ok =QInputDialog .getText (self ,"CSV File Name","Enter the name for the CSV file (without .csv):")
            if not ok or not csv_name :
                return 


            progress =QProgressDialog ("Saving analysis...","Cancel",0 ,100 ,self )
            progress .setWindowModality (Qt .WindowModal )
            progress .setWindowTitle ("Save Progress")
            progress .show ()


            plots_dir =os .path .join (save_dir ,"plots")
            os .makedirs (plots_dir ,exist_ok =True )


            self .save_csv (progress ,save_dir ,csv_name )


            self .save_plots (progress ,plots_dir )

            progress .setValue (100 )
            QMessageBox .information (self ,"Save Complete","Analysis has been saved successfully.")
        except Exception as e :
            error_msg =f"An error occurred while saving the analysis: {str (e )}"
            print (error_msg )
            QMessageBox .critical (self ,"Error",error_msg )

    def save_csv (self ,progress ,save_dir ,csv_name ):
        file_name =os .path .join (save_dir ,f"{csv_name }.csv")

        total_events =sum (len (events )for events in self .events_data .values ())

        with open (file_name ,'w',newline ='')as csvfile :
            writer =csv .writer (csvfile )
            writer .writerow (['Category','Number of Events','Percentage','Classifications','Number of Events','Percentage within Category','Percentage of Total Events'])

            row_count =sum (1 for _ in self .events_data .items ())
            for i ,(segment_count ,events )in enumerate (self .events_data .items ()):
                category_events =len (events )
                category_percentage =(category_events /total_events )*100 


                writer .writerow ([
                f"{segment_count } segments",
                category_events ,
                f"{category_percentage :.2f}%",
                "","","",""
                ])


                classifications ={}
                for event in events :
                    if self .reclassify_checkbox .isChecked ()and event [3 ]!=event [5 ]:
                        classification_key =f"{event [3 ]} -> {event [5 ]}"
                    else :
                        classification_key =event [3 ]
                    classifications [classification_key ]=classifications .get (classification_key ,0 )+1 

                for classification ,count in classifications .items ():
                    writer .writerow ([
                    "","","",
                    classification ,
                    count ,
                    f"{(count /category_events )*100 :.2f}%",
                    f"{(count /total_events )*100 :.2f}%"
                    ])

                progress .setValue (int ((i +1 )/row_count *50 ))


    def save_plots (self ,progress ,plots_dir ):

        self .save_figure (self .all_events_histogram_canvas ,os .path .join (plots_dir ,"all_events_histogram.png"))
        self .save_figure (self .all_events_scatter_canvas ,os .path .join (plots_dir ,"all_events_scatter.png"))


        self .save_figure (self .selected_classifications_histogram_canvas ,os .path .join (plots_dir ,"selected_events_histogram.png"))
        self .save_figure (self .selected_classifications_scatter_canvas ,os .path .join (plots_dir ,"selected_events_scatter.png"))


        total_categories =len (self .events_data )
        total_classifications =sum (len (set (event [3 ]for event in events ))for events in self .events_data .values ())
        total_items =total_categories +total_classifications 

        current_item =0 

        for segment_count ,events in self .events_data .items ():
            category_dir =os .path .join (plots_dir ,f"category_{segment_count }_segments")
            os .makedirs (category_dir ,exist_ok =True )


            self .plot_category_histogram (segment_count ,events )
            self .save_figure (plt .gcf (),os .path .join (category_dir ,f"category_{segment_count }_segments_histogram.png"))
            plt .close ()

            self .plot_category_scatter (segment_count ,events )
            self .save_figure (plt .gcf (),os .path .join (category_dir ,f"category_{segment_count }_segments_scatter.png"))
            plt .close ()

            current_item +=1 
            progress .setValue (int (50 +(current_item /total_items )*50 ))


            classifications =set (event [3 ]for event in events )
            for classification in classifications :
                classification_events =[event for event in events if event [3 ]==classification ]

                self .plot_classification_histogram (classification ,[event [0 ]for event in classification_events ])
                self .save_figure (plt .gcf (),os .path .join (category_dir ,f"classification_{classification }_histogram.png"))
                plt .close ()

                self .plot_classification_scatter (classification ,[event [0 ]for event in classification_events ])
                self .save_figure (plt .gcf (),os .path .join (category_dir ,f"classification_{classification }_scatter.png"))
                plt .close ()

                current_item +=1 
                progress .setValue (int (50 +(current_item /total_items )*50 ))

    def save_figure (self ,canvas ,filename ):
        if isinstance (canvas ,FigureCanvas ):
            canvas .figure .savefig (filename )
        elif isinstance (canvas ,plt .Figure ):
            canvas .savefig (filename )
        else :
            print (f"Unsupported figure type for {filename }")


    def plot_classification_histogram (self ,classification ,event_ids ):
        all_mean_diffs =[]
        for event_id in event_ids :
            key =f'SEGMENT_INFO_{event_id }_segment_mean_diffs'
            if key in self .data :
                mean_diffs =self .data [key ]
                all_mean_diffs .extend (mean_diffs )

        plt .figure ()
        plt .hist (all_mean_diffs ,bins ='auto',edgecolor ='black')
        plt .title (f'Segment Mean Diffs for Classification {classification }\n(Events: {len (event_ids )})')
        plt .xlabel ('Mean Diff')
        plt .ylabel ('Frequency')

    def plot_classification_scatter (self ,classification ,event_ids ):
        max_mean_diffs =[]
        event_widths =[]
        missing_data_count =0 
        data_keys =set (self .data .keys ())




        for event_id in event_ids :
            mean_diffs_key =f'SEGMENT_INFO_{event_id }_segment_mean_diffs'
            segment_widths_key =f'SEGMENT_INFO_{event_id }_segment_widths_time'

            if mean_diffs_key in self .data and segment_widths_key in self .data :
                mean_diffs =self .data [mean_diffs_key ]
                segment_widths =self .data [segment_widths_key ]
                if len (mean_diffs )>0 and len (segment_widths )>0 :
                    max_mean_diffs .append (np .max (np .abs (mean_diffs )))
                    event_widths .append (np .sum (segment_widths ))
                else :
                    missing_data_count +=1 
                    print (f"Warning: Empty data for event_id {event_id }")
            else :
                missing_data_count +=1 
                print (f"Warning: Missing data for event_id {event_id }")
                print (f"Keys not found: {mean_diffs_key if mean_diffs_key not in self .data else ''}, "
                f"{segment_widths_key if segment_widths_key not in self .data else ''}")

        if len (max_mean_diffs )>0 and len (event_widths )>0 :
            plt .figure ()
            plt .scatter (np .log (np .array (event_widths )*1e3 ),max_mean_diffs )
            plt .title (f'Max Segment Mean Diffs vs log(Event Width) for Classification {classification }\n'
            f'(Events: {len (event_ids )}, Plotted: {len (max_mean_diffs )})')
            plt .xlabel ('log(Δt (ms))')
            plt .ylabel ('|ΔI|')
            if missing_data_count >0 :
                plt .figtext (0.5 ,0.01 ,f"Warning: {missing_data_count } events had missing or invalid data",
                wrap =True ,horizontalalignment ='center',fontsize =8 )
        else :
            print (f"No data to plot for classification {classification }. "
            f"Total events: {len (event_ids )}, Missing data: {missing_data_count }")

            plt .figure ()
            plt .text (0.5 ,0.5 ,f"No data to plot for classification {classification }\n"
            f"Total events: {len (event_ids )}, Missing data: {missing_data_count }",
            horizontalalignment ='center',verticalalignment ='center')
            plt .axis ('off')




    def plot_category_histogram (self ,segment_count ,events ):
        all_mean_diffs =[]
        for event in events :
            all_mean_diffs .extend (event [1 ])

        plt .figure ()
        plt .hist (all_mean_diffs ,bins ='auto',edgecolor ='black')
        plt .title (f'Segment Mean Diffs for Category: {segment_count } segments\n(Events: {len (events )})')
        plt .xlabel ('Mean Diff')
        plt .ylabel ('Frequency')

    def plot_category_scatter (self ,segment_count ,events ):
        max_mean_diffs =[np .max (np .abs (event [1 ]))for event in events ]
        event_widths =[np .sum (event [2 ])for event in events ]

        plt .figure ()
        plt .scatter (np .log (np .array (event_widths )*1e3 ),max_mean_diffs )
        plt .title (f'Max Segment Mean Diffs vs log(Event Width) for Category: {segment_count } segments\n(Events: {len (events )})')
        plt .xlabel ('log(Δt (ms))')
        plt .ylabel ('|ΔI|')

if __name__ =="__main__":
    app =QApplication (sys .argv )
    app .setStyle (QStyleFactory .create ('Fusion'))
    palette =QPalette ()
    palette .setColor (QPalette .ColorRole .Window ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .WindowText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Base ,QColor (25 ,25 ,25 ))
    palette .setColor (QPalette .ColorRole .AlternateBase ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .ToolTipBase ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .ToolTipText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Text ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Button ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .ButtonText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .BrightText ,Qt .GlobalColor .red )
    palette .setColor (QPalette .ColorRole .Link ,QColor (42 ,130 ,218 ))
    palette .setColor (QPalette .ColorRole .Highlight ,QColor (42 ,130 ,218 ))
    palette .setColor (QPalette .ColorRole .HighlightedText ,Qt .GlobalColor .black )
    app .setPalette (palette )
    mainWin =SDAnalysisApp ()
    mainWin .showMaximized ()
    sys .exit (app .exec ())



def _site_packages (prefix ,real_prefix ,global_site_packages ):
    import os 
    import site 
    import sys 

    paths =[]

    paths .append (
    os .path .join (
    prefix ,"lib","python%d.%d"%(sys .version_info [:2 ]),"site-packages"
    )
    )
    if os .path .join (".framework","")in os .path .join (prefix ,""):
        home =os .environ .get ("HOME")
        if home :
            paths .append (
            os .path .join (
            home ,
            "Library",
            "Python",
            "%d.%d"%(sys .version_info [:2 ]),
            "site-packages",
            )
            )





    sys .__egginsert =len (sys .path )

    for path in paths :
        site .addsitedir (path )




    sys .__egginsert =len (sys .path )

    if global_site_packages :
        site .addsitedir (
        os .path .join (
        real_prefix ,
        "lib",
        "python%d.%d"%(sys .version_info [:2 ]),
        "site-packages",
        )
        )



import sys 


def _included_subpackages (packages ):
    for _pkg in packages :
        pass 


class Finder (object ):
    def find_spec (self ,fullname ,path ,target =None ):
        if fullname in _path_hooks :
            from importlib .machinery import ModuleSpec ,SourcelessFileLoader ,SourceFileLoader 
            from importlib .util import spec_from_loader 
            import os 
            pkg_dir =os .path .join (
            os .environ ["RESOURCEPATH"],"lib","python%d.%d"%(sys .version_info [:2 ])
            )
            init_path =os .path .join (pkg_dir ,fullname ,"__init__.py")
            if os .path .exists (init_path ):
                loader =SourceFileLoader (fullname ,init_path )
            else :
                loader =SourcelessFileLoader (fullname ,init_path +"c")

            return spec_from_loader (fullname ,loader )


    def find_module (self ,fullname ,path =None ):
        if fullname in _path_hooks :
            return Loader ()


class Loader (object ):
    def load_module (self ,fullname ):
        import imp 
        import os 

        pkg_dir =os .path .join (
        os .environ ["RESOURCEPATH"],"lib","python%d.%d"%(sys .version_info [:2 ])
        )
        return imp .load_module (
        fullname ,None ,os .path .join (pkg_dir ,fullname ),("","",imp .PKG_DIRECTORY )
        )


sys .meta_path .insert (0 ,Finder ())
